//======================================================================
// Solving Parametric Linear System Q(2,p)x = b(p) of specified dimension
// goto /CToolbox, modify PRJ1 in Makefile, execute: make clean all
// goto /CToolbox/Programs, execute: ./par_exQ2p 
//======================================================================


#include <iostream>
#include <string>
#include <parlinsysSparse.hpp>  // Parametric Linear System Solver with sparse coefficient matrices
#include "parlinsys.hpp"        // Parametric Linear System Solver with full coefficient matrices
#include <ivector.hpp>      
#include "timer.hpp"
#include <vector>

using namespace std;
using namespace cxsc;

int main() {
	int n, i, j, k, Err;
	real tol, Eps;

 	cout << SetPrecision(10, 16) << Scientific << endl; //OutputFormat

	cout << "Outer bounds for the solution set hull of Q(2,p)x = b(p)" << endl;

	// Input data:
	do { 
	  cout << "Enter the dimension of the system: ";
	  cin >> n;
	  cout << endl;
	} while(n <= 0);

        cout << "Generating matrix..." << endl;


        //********* Start of Q(2,p) matrix generation *********//

	srmatrix Ap(n*(n+1), n, 0), bp(n, n+1); 	
        vector<srmatrix> Apv;
        Apv.reserve(n+1);

	ivector p(n+1);			
	ivector xx;         		

        vector<int> indi;
        vector<int> indj;
        vector<real> xval;

        indi.reserve(n*n);
        indj.reserve(n*n);
        xval.reserve(n*n);
	
        for(i=1 ; i<=n ; i++) {
          for(j=1 ; j<=n ; j++) {
            if(i != j+2 && i>j) {
              indi.push_back(i-1);
              indj.push_back(j-1);
              xval.push_back(1.0);
            }

            if(i <= j) {
              indi.push_back(j*n+i-1);
              indj.push_back(j-1);
              xval.push_back(1.0);
            }
          }
        }

        intvector col(indi.size());
        intvector row(indj.size());
        rvector   val(xval.size());

        for(i=1 ; i<=indi.size() ; i++) {
          col[i] = indi[i-1];
          row[i] = indj[i-1];
          val[i] = xval[i-1];
        }

        indi.clear(); indi.reserve(0);
        indj.clear(); indj.reserve(0);
        xval.clear(); xval.reserve(0);

        Ap = srmatrix(n*(n+1), n, VecLen(val), col, row, val);

        //******** End of matrix generation **********//

        //For sparse solve: vector of coefficient matrices
        for(int i=0 ; i<n+1 ; i++) {
          Apv.push_back( srmatrix( Ap(i*n+1,(i+1)*n,1,n) ) );
        }

 	// Generating bp
 	for(i = 2; i <= n+1; i++)  { 
          //bp[Col(i)] = bp[Col(1)];
 	  j = i-1;
	  bp[Col(i)][j] = 1;
 	}

	tol = 0.5; 
	cout << "n= " << n << " Tol= " << tol << endl;

	p[1] = interval(1);	 // Initialize 1st element of the interval vector	
	for(i=2; i<=n+1; i++) p[i] = interval(i - i*tol/100, i + i*tol/100);


        rmatrix ApFull(Ap), bpFull(bp);

        double start,end;

        cout << endl << endl;

        cout << "Computation using ParLinSys with full matrices" << endl;
        start = GetTime();
        ParLinSolve(ApFull, bpFull, p, xx, true, Err, 0);
        end = GetTime();
        cout << end-start << endl;
	if(!Err) 
          cout << "Verified solution outer enclosure found in (last 10 elements):" << endl << xx(n-9,n) << endl << "Time: " << end-start << endl;
        else
          cout << ParLinSolveErrMsg(Err) << endl;

        cout << endl << endl;

        cout << "Computation using ParLinSys with sparse matrices" << endl;
        start = GetTime();
        ParLinSolve(Apv, bp, p, xx, true, Err);
        end = GetTime();

        if(!Err) 
           cout << "Verified solution outer enclosure found in (last 10 elements):" << endl << xx(n-9,n) << endl << "Time: " << end-start << endl; 
	else
	  cout << ParLinSolveSparseErrMsg(Err) << endl;	

        cout << endl << endl;

	return 0;
}

