//============================================================================ 
// 
//		 Program/Module "Parametric Linear Solver" 
// 
// 
//	  Copyright (c) 2003-2010   Evgenija Popova, Michael Zimmer
// 
// This program/module is free software for non-commercial use. 
// For details on theory  see the papers 
// 
// S. Rump: Verification methods for dense and sparse systems of equations, 
// in: Topics in Validated Computations, ed. J. Herzberger, Elsevier Science 
// B.V., 1994,	pp. 63-135. 
//
// E. Popova, W. Kraemer: Parametric Fixed-point Iteration Implemented in 
// C-XSC,  Universitaet Wuppertal, Preprint BUW - WRSWT, 2003.
// 
// This program/module is distributed WITHOUT ANY WARRANTY. 
// 
//============================================================================ 
//----------------------------------------------------------------------------
// File: parlinsysSparse (header)
//
// Purpose: Computation of a verified inclusion for the parametric solution set
// of a square parametric linear system of equations A(p)*x = b(p) with full
// parametric matrix A(p) and parametric right-hand side b(p) which elements are
// affine-linear combinations of k given interval parameters. The coefficient
// matrices are sparse.
//
// Global functions:
//
//    ParLinSolve()	          : to get a verified enclosure of the solution
//    ParLinSolveSparseErrMsg()   : to get an error message text
//
//----------------------------------------------------------------------------
#ifndef __PARLINSYS_SPARSECOEFF_HPP
#define __PARLINSYS_SPARSECOEFF_HPP

#include <scimatrix.hpp>   // Real sparse matrix/vector arithmetic
#include <civector.hpp>	   // Interval vector arithmetic
#include <vector>

char* ParLinSolveSparseErrMsg ( int );

//Real system, with inner enclosure
void  ParLinSolve (std::vector<cxsc::srmatrix>&, cxsc::srmatrix&, cxsc::ivector&, cxsc::ivector&, bool, cxsc::ivector&, int&, int=2);
//Real system, only outer enclosure
void  ParLinSolve (std::vector<cxsc::srmatrix>&, cxsc::srmatrix&, cxsc::ivector&, cxsc::ivector&, bool, int&, int=2);
//Complex system with complex parameters
void  ParLinSolve (std::vector<cxsc::scmatrix>&, cxsc::scmatrix&, cxsc::civector&, cxsc::civector&, bool, int&, int=2);
//System with real coefficients and complex parameters
void  ParLinSolve (std::vector<cxsc::srmatrix>&, cxsc::srmatrix&, cxsc::civector&, cxsc::civector&, bool, int&, int=2);

#endif // end __PARLINSYS
